// --------------------------------------------------------------------
// Projet POBOT Carte CNP pour la Coupe 2006.                  			---
// Fonctions Srie. Version 2.0.                                   	---
//                                                                  ---
// Cr le 27 Novembre 2005 par Spinelli Gilles.                    ---
// --------------------------------------------------------------------

#include "CarteCNP.h"


// ----- Dclaration des Constantes Globales -----


// ----- Dclaration des Variables Globales -----
volatile unsigned char      	cSerial_NbrRec;
unsigned char	            		szSerial_BufferRec[SERIAL_MAXI_BUFFERREC+1], szSerial_BufferSend[SERIAL_MAXI_BUFFERSEND+1];

volatile struct structFlags 	sSerial_Flags;
#define bSerial_FinalSend         sSerial_Flags.bit4

// ----- Dclaration des Constantes Locales. -----
// --- Table de Dbit Srie pour Quartz 8 MHz & U2X=0. Mais 76.8Kbps est  6%  ---
//flash unsigned int szBaudRate_Tab[8]={25,416,207,103,51,25,12,6};

// -- Table de Dbit Srie pour Quartz 16 MHz & U2X=0 --
const unsigned int szBaudRate_Tab[8]={25,832,416,207,103,51,25,12};


// ----- Dclarations des Variables Locales. -----
unsigned int	            	iSerial_CurrentTimeOut;
unsigned char               cSerial_NbrSend, cSerial_PtrSend, cSerial_NbrTransmit;
unsigned char								cSerial_TypeStop, cSerial_DataStop;



// ------------------------------------------------------------
// ----- Fonctions Srie sur Interruption.                -----
// ------------------------------------------------------------

// ----- Interruption Reception UART . -----
SIGNAL (SIG_USART_RECV)
{
  static unsigned char	cData;
  
  // -- Prends l'octet sur UART & le stocke en mmoire. --
  cData=UDR;;
  szSerial_BufferRec[cSerial_NbrRec++]=cData;

  // -- Teste si 1 Caractre. --
  if (!bSerial_BeginRec)
  	{ bSerial_BeginRec = TRUE; }
  bSerial_EndRec = FALSE;

  // -- Teste si dpasse Taille relle Buffer. Dans ce cas, Rception finie. --
  if (cSerial_NbrRec>SERIAL_MAXI_BUFFERREC)
  { 
  	cSerial_NbrRec = SERIAL_MAXI_BUFFERREC; 
  	bSerial_EndRec = TRUE; 
  }
  
	// -- Teste si fin de Trame Utilisateur --
	if (cSerial_TypeStop & FRAME_STOP_TIMEOUT)
	{
  	// -- MAJ Timer TimeOut --
  	iTimer_CptS = (unsigned int)(cSerial_DataStop);
  	bTimer_FlagS = FALSE;
	}
	else
	{
		if (cSerial_TypeStop & FRAME_STOP_CARACT)
		{
			if (cData == cSerial_DataStop)
				{ bSerial_EndRec = TRUE; }
		}
		else
		{
			if (cSerial_NbrRec>=cSerial_DataStop)
				{ bSerial_EndRec = TRUE; }
		} 	
	}
}



// ----- Dernire Interruption UART Emission. -----
SIGNAL (SIG_USART_TRANS)
{
	UCSRB = UCSRB & ~(_BV(TXCIE)); 
	bSerial_EndSend = TRUE; 
	MAX_RECEPTION();
}

// ----- Interruption UART Emission RS232. -----
SIGNAL (SIG_USART_DATA)
{
  // -- Prends l'octet de la mmoire & le stocke sur UART. --
  UDR=szSerial_BufferSend[cSerial_PtrSend++];
  
  // -- Si fin de donnes, attend fin IT Emission UART. --
  if (cSerial_PtrSend == cSerial_NbrSend)
  {
    bSerial_FinalSend = TRUE; 
    UCSRB = UCSRB & ~(_BV(UDRIE)); 
    UCSRB = UCSRB | _BV(TXCIE); 
  }
}


// ------------------------------------------------------------
// ----- Serial High Level Functions. 		          -----
// ------------------------------------------------------------

// ----- Initialisation de la liaison srie -----
void Serial_Init(unsigned char cBaudRate)
{
  unsigned int		iConfig;
  
  if (bSerial_BeginSend)
    { while (!bSerial_EndSend) { } }
  
  // -- Au cas ou erreur sur Dbit Srie --
  if (!cBaudRate || (cBaudRate > SERIAL_MAXI_BAUDRATE))
  	{cBaudRate = SERIAL_V19200; }
  
  UCSRA=0x00; UCSRB=0x00; 										// Indique Clock Bit x 1 .
  
  iConfig=szBaudRate_Tab[cBaudRate];					// Indique Dbit.
  UBRRH=(unsigned char)(iConfig/256);
  UBRRL=(unsigned char)(iConfig);

  UCSRC = _BV(URSEL) | _BV(UCSZ1) | _BV(UCSZ0);		// Indique Asynchrone/8/N/1.
  
  // -- Initialize Variables. ---
  cSerial_TypeStop=FRAME_STOP_TIMEOUT;   			// Define TimeOut comme fin de trame.
  cSerial_DataStop=SERIAL_DEFAULT_TIMEOUT;
  iTimer_CptS = 0x00;
  bTimer_FlagS = FALSE;

  cSerial_NbrRec=0;
  cSerial_NbrTransmit=0;

  bSerial_BeginRec = FALSE;
  bSerial_EndRec = FALSE;
  
  bSerial_BeginSend = FALSE;
  bSerial_FinalSend = FALSE;
  bSerial_EndSend = FALSE;
}


// -----  Dbut d'une Emission Srie. -----
void Serial_Send(unsigned char cNbrData)
{
  // -- Commence nouvelle transmission srie. --
  MAX_EMISSION();
  
  bSerial_BeginSend = TRUE; 
  bSerial_EndSend = FALSE; 
  cSerial_PtrSend = 0; 
  cSerial_NbrSend = cNbrData;
  
  // -- Passe UART en Emission sur IT. --
  UCSRB |= _BV(UDRIE) | _BV(TXEN);              // Indique Transmission On.
}



// -----  Initialisation de la Rception Srie.  -----
void Serial_InitRec(unsigned char cTypeStop, unsigned char cDataStop)
{
	// -- Stocke le type de fin de trame --
	cSerial_TypeStop = cTypeStop;
	cSerial_DataStop = cDataStop;
	iTimer_CptS = 0x00;										// Au cas ou, stoppe Timer Srie.
  bTimer_FlagS = FALSE;
	
  // -- Commence nouvelle rception srie. --
  cSerial_NbrRec = 0; 
  bSerial_BeginRec = FALSE; 
  bSerial_EndRec = FALSE;
    
  // -- Enable Rcepteur & IT Uart Reception. --
  UCSRB |= _BV(RXCIE) | _BV(RXEN);           				
}



// -----  Arret de la Rception Srie.  -----
void Serial_StopRec(void)
{
}

