// --------------------------------------------------------------------
// Projet POBOT Carte CNP.                                         	---
// Dclaration Fonctions & Variables Srie. Version 1.0.           	---
//                                                                  ---
// Cr le 27 Novembre 2005 par Spinelli Gilles.                    ---
// --------------------------------------------------------------------

// --------------------------------------------------
// ----- Dclaration des Constantes Globales.	-----
// --------------------------------------------------

// --- Constantes de Dfinition de la Longueur des Buffer Srie. ---
#define	  SERIAL_MAXI_BUFFERREC		200				// Buffer Srie Rception de 200 octets maximum.
#define	  SERIAL_MAXI_BUFFERSEND	200				// Buffer Srie Emission de 200 octets maximum.

// --- Constantes de Dfinition des Vitesses Srie ---
#define   SERIAL_V1200              1
#define   SERIAL_V2400              2
#define   SERIAL_V4800              3
#define   SERIAL_V9600              4
#define   SERIAL_V19200             5
#define   SERIAL_V38400             6
#define   SERIAL_V76800             7
#define   SERIAL_V115200            8

#define   SERIAL_MINI_BAUDRATE    	SERIAL_V1200   		
#define   SERIAL_MAXI_BAUDRATE    	SERIAL_V115200   		


// --- Constantes de Dfinition des Minimum Time Out Srie (en millisecondes) ---
#define   SERIAL_MINI_1200        	17
#define   SERIAL_MINI_2400        	9
#define   SERIAL_MINI_4800        	5
#define   SERIAL_MINI_9600        	3
#define   SERIAL_MINI_19200       	2
#define   SERIAL_MINI_38400       	2
#define   SERIAL_MINI_57600       	2
#define   SERIAL_MINI_115200      	2

#define	  SERIAL_MINI_TIMEOUT		SERIAL_MINI_115200
#define   SERIAL_DEFAULT_TIMEOUT  	5       		    // Time Out par dfaut : 5 ms. 
#define   SERIAL_MAXI_TIMEOUT     	100       		  	// Time Out Maximum : 100 ms. 


// --- Constantes de Dfinition des bits de Parit ---
#define   SERIAL_NO_PARITY        	1
#define   SERIAL_EVEN_PARITY      	2          		// Parit Paire.
#define   SERIAL_ODD_PARITY       	3          		// Parit Impaire.
                        

// --- Constantes de Dfinition du Nombre de bits par Data ---
#define   SERIAL_BYTE_7           	7
#define   SERIAL_BYTE_8           	8


// --- Constantes de Dfinition des bits de Stop ---
#define   SERIAL_STOP_1           	1
#define   SERIAL_STOP_2           	2


// --- Constantes de Dfinition du Type de Controle de Flux ---
#define   SERIAL_FLOW_HARD        	0
#define   SERIAL_FLOW_SOFT        	1
#define   SERIAL_FLOW_NONE        	2


// --- Constantes de Dfinition des bytes Contrle de Flux ---
#define   SERIAL_XON              	0x11
#define   SERIAL_XOFF             	0x13


// --- Constantes de Dfinition du Type d'arrt de Trame ---
#define   FRAME_STOP_TIMEOUT       	0x01
#define   FRAME_STOP_CARACT         0x02
#define   FRAME_STOP_LENGTH         0x04


// -----------------------------------------
// ----- Global Structure Declaration. -----
// -----------------------------------------

// -----------------------------------------
// ----- Global Variables Declaration. -----
// -----------------------------------------

// -- szSerialBufferRec est le Buffer utilis par la rception srie 							--
// -- szSerialBufferSend est le Buffer utilis par l'mission srie 							--
// -- cSerial_NbrRec est le nombre de caractres reus 														--
// -- bSerial_BeginRec indique que la rception srie a reu au moins 1 caractre.--
// -- bSerial_EndRec indique que la rception srie est finie.										--
// -- bSerial_BeginSend indique que l'mission srie a dmarr.										--
// -- bSerial_EndSend indique que l'mission srie est finie.											--

extern unsigned char	            szSerial_BufferRec[SERIAL_MAXI_BUFFERREC+1];
extern unsigned char							szSerial_BufferSend[SERIAL_MAXI_BUFFERSEND+1];
extern volatile unsigned char     cSerial_NbrRec;


extern volatile struct structFlags    sSerial_Flags;              
#define bSerial_BeginRec        sSerial_Flags.bit0
#define bSerial_EndRec          sSerial_Flags.bit1
#define bSerial_BeginSend  		  sSerial_Flags.bit2
#define bSerial_EndSend         sSerial_Flags.bit3


// -----------------------------------------
// ----- Global Functions Declaration. -----
// -----------------------------------------

// -- Serial_Init initialise la Liaison srie : Vitesse indique, 8 bits, pas de	--
// -- parit, 1 bit de stop. cBaudRate est dfini par les constantes SERIAL_Vxxxx	--

// -- Serial_InitRec dmarre la rception srie et dfini les conditions d'arret.	--
// -- Cette fonction ressort immdiatement, la gestion se faisant sur IT.					--
// -- La rception de donnes est finie lorsque bSerial_EndRec = TRUE.						--
// -- Les cSerial_NbrRec donnes reues sont stocke dans szSerial_BufferRec.			--
// -- cTypeStop indique le type d'arret : TimeOut, Caractre ou Longueur.					--
// -- cDataStop est la variable attache : dure du TimeOut (ms), Caractre(0-255)--
// -- ou Longueur (1-200).																												--

// -- Serial_Send envoie sur la liaison srie le contenu de szSerial_BufferSend.	--
// -- Cette fonction ressort immdiatement, la gestion se faisant sur IT.					--
// -- cNbrData indique le nombre d'octets  envoyer.															--
// -- L'mission des donnes est finie lorsque bSerial_EndSend = TRUE.						--

void            Serial_Init(unsigned char cBaudRate);

void            Serial_InitRec(unsigned char cTypeStop, unsigned char cDataStop);
void            Serial_StopRec(void);

void            Serial_Send(unsigned char cNbrData);



